@@ -6,8 +6,6 @@ import urlparse
 from OpenSSL import SSL
 from OpenSSL.crypto import load_certificate, FILETYPE_PEM
 
-from StringIO import StringIO
-
 from twisted.internet import reactor, ssl
 from twisted.internet.error import DNSLookupError, TimeoutError
 from twisted.internet.abstract import isIPAddress
@@ -15,15 +13,15 @@ from twisted.internet.defer import (
     Deferred,
     inlineCallbacks,
     )
-from twisted.internet.endpoints import TCP4ClientEndpoint
 from twisted.internet.protocol import (
     ClientCreator,
     DatagramProtocol,
     Protocol,
     )
 from twisted.protocols.memcache import MemCacheProtocol
-from twisted.web.client import Agent, FileBodyProducer, ProxyAgent
-from twisted.web.http_headers import Headers
+
+from txrequests import Session
+from requests.packages import urllib3
 
 from .check_impl import (
     add_check_prefix,
@@ -154,7 +152,7 @@ def do_udp_check(host, port, send, expect):
     else:
         ip = host
     deferred = Deferred()
-    protocol = UDPCheckProtocol(host, port, send, expect, deferred)
+    protocol = UDPCheckProtocol(ip, port, send, expect, deferred)
     reactor.listenUDP(0, protocol)
     try:
         yield deferred
@@ -190,31 +188,49 @@ def extract_host_port(url):
 def make_http_check(url, method='GET', expected_code=200, **kwargs):
     subchecks = []
     host, port, scheme = extract_host_port(url)
-    subchecks.append(make_tcp_check(host, port))
-    if scheme == 'https':
-        subchecks.append(make_ssl_check(host, port))
+    proxy_host = kwargs.get('proxy_host')
+    proxy_port = kwargs.get('proxy_port', 8000)
+
+    if proxy_host:
+        subchecks.append(make_tcp_check(proxy_host, proxy_port))
+    else:
+        subchecks.append(make_tcp_check(host, port))
 
     @inlineCallbacks
     def do_request():
-        proxy_host = kwargs.get('proxy_host')
+        proxies = {}
         if proxy_host:
-            proxy_port = kwargs.get('proxy_port', 8000)
-            endpoint = TCP4ClientEndpoint(reactor, proxy_host, proxy_port)
-            agent = ProxyAgent(endpoint)
-        else:
-            agent = Agent(reactor)
+            proxies['http'] = proxies['https']= '{}:{}'.format(
+                                                 proxy_host, proxy_port)
 
         headers = kwargs.get('headers')
-        if headers:
-            headers = Headers(headers)
         body = kwargs.get('body')
-        if body:
-            body = FileBodyProducer(StringIO(body))
+        disable_tls_verification = kwargs.get('disable_tls_verification',
+                                              False)
 
-        response = yield agent.request(method, url, headers, body)
-        if response.code != expected_code:
-            raise RuntimeError(
-                "Unexpected response code: {}".format(response.code))
+        if disable_tls_verification:
+            urllib3.disable_warnings()
+
+        args = {
+            'method': method,
+            'url': url,
+            'verify': not disable_tls_verification,
+        }
+        if headers:
+            args['headers'] = headers
+        if body:
+            args['data'] = body
+        if proxies:
+            args['proxies'] = proxies
+
+        with Session() as session:
+            request = session.request(**args)
+
+            response = yield request
+            if response.status_code != expected_code:
+                raise RuntimeError(
+                    "Unexpected response code: {}".format(
+                                               response.status_code))
 
     subchecks.append(make_check('http:{}'.format(url), do_request,
                      info='{} {}'.format(method, url)))
